@@ -24,12 +24,24 @@ fn main() {
     let identity = openstack::auth::Identity::from_env()
         .expect("Failed to create an identity provider from the environment");
     let session = openstack::Session::new(identity);
-    let servers = openstack::compute::v2::servers(&session).list()
-        .sort_by(openstack::Sort::Asc("access_ip_v4")).fetch()
-        .expect("Cannot list servers");
+    let server_manager = openstack::compute::v2::servers(&session);
+    let sorting = openstack::compute::v2::ServerSortKey::AccessIpv4;
+
+    let servers = server_manager.query()
+        .sort_by(openstack::Sort::Asc(sorting))
+        .fetch().expect("Cannot list servers");
+    println!("All servers:");
     for s in &servers {
         println!("ID = {}, Name = {}", s.id(), s.name());
     }
+
+    let active = server_manager.query()
+        .sort_by(openstack::Sort::Asc(sorting)).with_status("ACTIVE")
+        .fetch().expect("Cannot list servers");
+    println!("Only active servers:");
+    for s in &active {
+        println!("ID = {}, Name = {}", s.id(), s.name());
+    }
 }
 
 #[cfg(not(feature = "compute"))]
