@@ -14,21 +14,17 @@
 
 //! Base code for authentication.
 
-use std::fmt;
+use std::fmt::Debug;
 
 use hyper::{Client, Url};
-use time::PreciseTime;
 
 use super::super::{ApiResult, Session};
 
 
 /// Trait for authentication token implementations.
-pub trait Token: Clone + fmt::Debug + Send + fmt::Display + Into<String> {
-    /// A reference to token contents.
-    fn value(&self) -> &String;
-
-    /// Expiration time (if any).
-    fn expires_at(&self) -> Option<&PreciseTime>;
+pub trait Token: Clone + Debug + Into<String> {
+    /// Check whether the token needs refreshing right now.
+    fn needs_refresh(&self) -> bool;
 }
 
 /// Trait for an authentication method.
