@@ -66,13 +66,10 @@ extern crate hyper;
 extern crate hyper_rustls;
 #[macro_use]
 extern crate log;
-#[macro_use]
-extern crate mime;
 extern crate serde;
 #[macro_use]
 extern crate serde_derive;
 extern crate serde_json;
-extern crate time;
 
 #[cfg(test)] #[macro_use]
 extern crate yup_hyper_mock;
@@ -81,7 +78,6 @@ pub mod auth;
 mod common;
 #[cfg(feature = "compute")]
 pub mod compute;
-pub mod http;
 mod identity;
 pub mod service;
 mod session;
