@@ -16,18 +16,28 @@
 
 use std::marker::PhantomData;
 
-use hyper::{Get, Url};
-use hyper::header::Headers;
-use hyper::method::Method;
-use serde::Deserialize;
+use hyper::Url;
+use hyper::client::{Body, RequestBuilder as HyperRequestBuilder, Response};
+use hyper::header::{Header, HeaderFormat, Headers};
+pub use hyper::method::Method;
+use serde::{Deserialize, Serialize};
 use serde_json;
 
-use super::{ApiResult, ApiVersion, ApiVersionRequest, Session};
+use super::{ApiError, ApiResult, ApiVersion, ApiVersionRequest, Session};
 use super::auth::Method as AuthMethod;
-use super::http::AuthenticatedRequestBuilder;
+use super::identity::protocol::AuthTokenHeader;
 use super::utils;
 
 
+/// Request builder with authentication, error checking and JSON support.
+///
+/// Partly copies the interface of hyper::client::RequestBuilder.
+#[allow(missing_debug_implementations)]
+pub struct RequestBuilder<'a, A: AuthMethod + 'a> {
+    parent: &'a Session<A>,
+    inner: HyperRequestBuilder<'a>
+}
+
 /// Type of query parameters.
 #[derive(Clone, Debug)]
 pub struct Query(pub Vec<(String, String)>);
@@ -74,11 +84,80 @@ impl Query {
         Query(Vec::new())
     }
 
-    /// Add items to the query.
+    /// Add an item to the query.
     pub fn push<K, V>(&mut self, param: K, value: V)
             where K: Into<String>, V: ToString {
         self.0.push((param.into(), value.to_string()))
     }
+
+    /// Add a strng item to the query.
+    pub fn push_str<K, V>(&mut self, param: K, value: V)
+            where K: Into<String>, V: Into<String> {
+        self.0.push((param.into(), value.into()))
+    }
+}
+
+impl<'a, Auth: AuthMethod> RequestBuilder<'a, Auth> {
+    /// Wrap a request builder.
+    pub fn new(inner: HyperRequestBuilder<'a>, session: &'a Session<Auth>)
+            -> RequestBuilder<'a, Auth> {
+        RequestBuilder {
+            parent: session,
+            inner: inner
+        }
+    }
+
+    /// Send this request.
+    pub fn send(self) -> ApiResult<Response> {
+        let resp = try!(self.send_unchecked());
+        if resp.status.is_success() {
+            Ok(resp)
+        } else {
+            Err(ApiError::HttpError(resp.status, resp))
+        }
+    }
+
+    /// Send this request without checking on status code.
+    pub fn send_unchecked(self) -> ApiResult<Response> {
+        let token = try!(self.parent.auth_token());
+        let hdr = AuthTokenHeader(token.into());
+        self.inner.header(hdr).send().map_err(From::from)
+    }
+
+    /// Send this request and parse JSON response on success.
+    pub fn fetch_json<T: Deserialize>(self) -> ApiResult<T> {
+        let resp = try!(self.send());
+        serde_json::from_reader(resp).map_err(From::from)
+    }
+
+    /// Add body to the request.
+    pub fn body<B: Into<Body<'a>>>(self, body: B)
+            -> RequestBuilder<'a, Auth> {
+        RequestBuilder {
+            inner: self.inner.body(body),
+            .. self
+        }
+    }
+
+    /// Add additional headers to the request.
+    pub fn headers(self, headers: Headers)
+            -> RequestBuilder<'a, Auth> {
+        RequestBuilder {
+            inner: self.inner.headers(headers),
+            .. self
+        }
+    }
+
+    /// Add an individual header to the request.
+    ///
+    /// Note that X-Auth-Token is always overwritten with a token in use.
+    pub fn header<H: Header + HeaderFormat>(self, header: H)
+            -> RequestBuilder<'a, Auth> {
+        RequestBuilder {
+            inner: self.inner.header(header),
+            .. self
+        }
+    }
 }
 
 impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
@@ -103,7 +182,7 @@ impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
 
     /// Make an HTTP request to the given service.
     pub fn request<P>(&'session self, method: Method, path: P, query: Query)
-            -> ApiResult<AuthenticatedRequestBuilder<'session, Auth>>
+            -> ApiResult<RequestBuilder<'session, Auth>>
             where P: IntoIterator, P::Item: AsRef<str> {
         let url = try!(self.get_endpoint(path, query));
         let headers = self.session.service_headers::<Srv>();
@@ -112,15 +191,53 @@ impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
         Ok(self.session.raw_request(method, url).headers(headers))
     }
 
-    /// Make a GET request.
-    pub fn http_get<P, Res>(&self, path: P, query: Query) -> ApiResult<Res>
+    /// Make an HTTP request with JSON body and JSON response.
+    pub fn json<P, Req, Res>(&self, method: Method, path: P, query: Query,
+                             body: &Req) -> ApiResult<Res>
+            where Req: Serialize, Res: Deserialize,
+            P: IntoIterator, P::Item: AsRef<str> {
+        let str_body = try!(serde_json::to_string(body));
+        let request = try!(self.request(method, path, query));
+        request.body(&str_body).fetch_json()
+    }
+
+    /// Make a GET request returning a JSON.
+    pub fn get_json<P, Res>(&self, path: P, query: Query) -> ApiResult<Res>
             where Res: Deserialize, P: IntoIterator, P::Item: AsRef<str> {
-        let request = try!(self.request(Get, path, query));
-        let resp = try!(request.send());
-        serde_json::from_reader(resp).map_err(From::from)
+        try!(self.request(Method::Get, path, query)).fetch_json()
+    }
+
+    /// Make a POST request sending and returning a JSON.
+    pub fn post_json<P, Req, Res>(&self, path: P, query: Query, body: &Req)
+            -> ApiResult<Res> where Req: Serialize, Res: Deserialize,
+            P: IntoIterator, P::Item: AsRef<str> {
+        self.json(Method::Post, path, query, body)
+    }
+
+    /// Make a POST request sending and returning a JSON.
+    pub fn put_json<P, Req, Res>(&self, path: P, query: Query, body: &Req)
+            -> ApiResult<Res> where Req: Serialize, Res: Deserialize,
+            P: IntoIterator, P::Item: AsRef<str> {
+        self.json(Method::Put, path, query, body)
+    }
+
+    /// Make a PATCH request sending and returning a JSON.
+    pub fn patch_json<P, Req, Res>(&self, path: P, query: Query, body: &Req)
+            -> ApiResult<Res> where Req: Serialize, Res: Deserialize,
+            P: IntoIterator, P::Item: AsRef<str> {
+        self.json(Method::Patch, path, query, body)
+    }
+
+    /// Make a DELETE request.
+    pub fn delete<P>(&self, path: P, query: Query) -> ApiResult<Response>
+            where P: IntoIterator, P::Item: AsRef<str> {
+        try!(self.request(Method::Delete, path, query)).send()
     }
 }
 
+impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
+        Copy for ServiceWrapper<'session, Auth, Srv> {}
+
 impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
         Clone for ServiceWrapper<'session, Auth, Srv> {
     fn clone(&self) -> ServiceWrapper<'session, Auth, Srv> {
