@@ -14,14 +14,15 @@
 
 //! Various utilities.
 
-use std::cell::RefCell;
+use std::cell::{Ref, RefCell};
+use std::collections::HashMap;
+use std::hash::Hash;
 
 use hyper::Client;
 #[cfg(feature = "tls")]
 use hyper::net::HttpsConnector;
 #[cfg(feature = "tls")]
 use hyper_rustls::TlsClient;
-use uuid::Uuid;
 
 use super::ApiResult;
 
@@ -43,6 +44,10 @@ pub fn http_client() -> Client {
 #[derive(Debug, Clone)]
 pub struct ValueCache<T: Clone>(RefCell<Option<T>>);
 
+/// Cached map of values.
+#[derive(Debug, Clone)]
+pub struct MapCache<K: Hash + Eq, V: Clone>(RefCell<HashMap<K, V>>);
+
 
 impl<T: Clone> ValueCache<T> {
     /// Create a cache.
@@ -70,36 +75,38 @@ impl<T: Clone> ValueCache<T> {
     }
 }
 
+impl<K: Hash + Eq, V: Clone> MapCache<K, V> {
+    /// Create a cache.
+    pub fn new() -> MapCache<K, V> {
+        MapCache(RefCell::new(HashMap::new()))
+    }
 
-/// Something that can be converted to an ID.
-pub trait IntoId {
-    /// Convert a value into an ID.
-    fn into_id(self) -> String;
-}
+    /// Ensure the value is present in the cache.
+    pub fn ensure_value<F>(&self, key: K, default: F) -> ApiResult<()>
+            where F: FnOnce(&K) -> ApiResult<V> {
+        if self.0.borrow().contains_key(&key) {
+            return Ok(());
+        }
 
-impl IntoId for Uuid {
-    fn into_id(self) -> String {
-        self.to_string()
+        let new = try!(default(&key));
+
+        let _ = self.0.borrow_mut().insert(key, new);
+        Ok(())
+    }
+
+    /// Get a reference to the value.
+    ///
+    /// Borrows the inner RefCell.
+    pub fn get_ref(&self, key: &K) -> Option<Ref<V>> {
+        let map = self.0.borrow();
+        if map.contains_key(key) {
+            Some(Ref::map(map, |m| m.get(&key).unwrap()))
+        } else {
+            None
+        }
     }
 }
 
-impl IntoId for String {
-    fn into_id(self) -> String {
-        self
-    }
-}
-
-impl<'a> IntoId for &'a String {
-    fn into_id(self) -> String {
-        self.clone()
-    }
-}
-
-impl<'a> IntoId for &'a str {
-    fn into_id(self) -> String {
-        String::from(self)
-    }
-}
 
 pub mod url {
     //! Handy primitives for working with URLs.
@@ -123,7 +130,8 @@ pub mod url {
 
     #[inline]
     #[allow(unused_results)]
-    pub fn extend(mut url: Url, segments: &[&str]) -> Url {
+    pub fn extend<I>(mut url: Url, segments: I) -> Url
+            where I: IntoIterator, I::Item: AsRef<str> {
         url.path_segments_mut().unwrap().pop_if_empty().extend(segments);
         url
     }
